/***************************************************************************
 *   Copyright (C) 2011 by Azrael   *
 *      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <ds.h>

#ifndef FALSE
#define FALSE (0)
#endif

#ifndef TRUE
#define TRUE (!FALSE)
#endif

list_compare_t func_compare(const void *a, const void *b)
{
	if (((int) a) > ((int) b))
	{
		return TRUE;
	}
	return FALSE;
}


struct list *list_new()
{
	struct list *list_new;
	list_new = malloc(sizeof(struct list));
	if (list_new == NULL) {
		return NULL;
	};
	return list_new;
}

void list_free(struct list **list)
{
	free(*list);
	*list = NULL;
	return;
}

void list_add_data(struct list *list, void *data)
{
	list->tail->data = data;
	return;
}

void list_del_data(struct list *list, void *data)
{
	struct list_item *h;
	h = list->head;
	while(h != NULL){
		if (h->data == data){
			h->data = NULL;
		}
		h = h->next;
	}
	return;
}

void list_add (struct list *list, struct list_item *item)
{
	item->next = list->head;
	list->head->prev = item;
	item->prev = NULL; //надо или нет?
	list->head = item;
	return;
}

void list_add_tail(struct list *list, struct list_item *item)
{
	item->prev = list->tail;
	list->tail->next = item;
	item->next = NULL;
	list->tail = item;
	return;
}

struct list_item *list_unlink(struct list *list, struct list_item *item)
{
	struct list_item *h;
	h = list->head;
	while (h != item)
	{
		if (h->next != NULL)
		{
			h = h->next;
		} else {
			return NULL;
		}	
	}
	if (h->next == NULL)
	{
		h = h->prev;
		free(h->next);
		h->next = NULL;
	} else {
		struct list_item *w;
		struct list_item *h1;
		h1 = h;
		w = h->prev;
		w->next = h->next;
		h = h->next;
		h->prev = w;
		return h1;
	}
}

struct list_item *list_nth(struct list *list, unsigned int nth)
{
	if (nth == 0) 
	{
		return list->head;
	}
	struct list_item *h;
	h = list->head;
	int i = 0;
	for (i;i != nth;i++){
		h = h->next;
		if (h == NULL) 
		{
			return NULL;
		}
	}
	return h;
}

void list_sort(struct list *list, list_compare_t compare)
{
	struct list_item *h;
	struct list_item *w;
	h = list->head;
	int z=1;
	int res;
	while (z != 0) {
		z = 1;
		while (h->next != NULL) 
		{
			res = (*compare)(h->data,h->next->data);
			if (res != 0) 
			{
				z = 0;
				w = h->next;
				w->prev = h->prev;
				h->next = h->prev;
				h->prev = w;
				w->next = h;
			}
		}
		h = list->head;
	}
	return;
}

size_t list_count(struct list *list)
{
	size_t n=0;
	struct list_item *h;
	h = list->head;
	while(h != NULL){
		h = h->next;
		++n;
	}
	return n;
}

void list_traverse (struct list *list, list_callback_t func, void *opaque)
{
	struct list_item *iter;
	int res;
	iter = list->head;
	for (iter; iter != NULL; iter = iter->next) {
		res = (*func)(iter, opaque);
		if (res)
		{
			return;
		}
	}
}

int main(int argc, char *argv[])
{
  printf("Hello, world!\n");

  return EXIT_SUCCESS;
}
